/*! markmap-lib2 v0.6.1 | MIT License */
'use strict';

var d3 = require('d3');
var d3Flextree = require('d3-flextree');

const uniqId = Math.random().toString(36).slice(2, 8);
let globalIndex = 0;

function getId() {
  globalIndex += 1;
  return `mm-${uniqId}-${globalIndex}`;
}

function walkTree(tree, callback, key = 'c') {
  const walk = (item, parent) => callback(item, () => {
    var _item$key;

    (_item$key = item[key]) == null ? void 0 : _item$key.forEach(child => {
      walk(child, item);
    });
  }, parent);

  walk(tree);
}

function linkWidth(nodeData) {
  const data = nodeData.data;
  return Math.max(6 - 2 * data.d, 1.5);
}

function adjustSpacing(tree, spacing) {
  walkTree(tree, (d, next) => {
    d.ySizeInner = d.ySize - spacing;
    d.y += spacing;
    next();
  }, 'children');
}

function arrayFrom(arrayLike) {
  const array = [];

  for (let i = 0; i < arrayLike.length; i += 1) {
    array.push(arrayLike[i]);
  }

  return array;
}

function childSelector(filter) {
  if (typeof filter === 'string') {
    const tagName = filter;

    filter = el => el.tagName === tagName;
  }

  const filterFn = filter;
  return function selector() {
    let nodes = arrayFrom(this.childNodes);
    if (filterFn) nodes = nodes.filter(node => filterFn(node));
    return nodes;
  };
}

function addClass(className, ...rest) {
  const classList = (className || '').split(' ').filter(Boolean);
  rest.forEach(item => {
    if (item && classList.indexOf(item) < 0) classList.push(item);
  });
  return classList.join(' ');
}

function markmap(svg, data, opts) {
  svg = svg.datum ? svg : d3.select(svg);
  const styleNode = svg.append('style');
  const zoom = d3.zoom().on('zoom', handleZoom);
  const svgNode = svg.node();
  const options = {
    duration: 500,
    nodeFont: '300 16px/20px sans-serif',
    nodeMinHeight: 16,
    spacingVertical: 5,
    spacingHorizontal: 80,
    autoFit: false,
    fitRatio: 0.95,
    color: d3.scaleOrdinal(d3.schemeCategory10),
    colorDepth: 0,
    paddingX: 8,
    ...opts
  };
  const state = {
    id: options.id || getId()
  };
  const g = svg.append('g').attr('class', `${state.id}-g`);
  updateStyle();

  if (data) {
    setData(data);
    fit(); // always fit for the first render
  }

  svg.call(zoom);
  return {
    setData,
    setOptions,
    fit
  };

  function getStyleContent() {
    const {
      style
    } = options;
    const styleText = style ? style(state.id) : `\
.${state.id} a { color: #0097e6; }
.${state.id} a:hover { color: #00a8ff; }
.${state.id}-g > path { fill: none; }
.${state.id}-fo > div { font: ${options.nodeFont}; white-space: nowrap; }
.${state.id}-fo > div > code { padding: .2em .4em; font-size: calc(1em - 2px); color: #555; background-color: #f0f0f0; border-radius: 2px; }
.${state.id}-fo > div > del { text-decoration: line-through; }
.${state.id}-fo > div > em { font-style: italic; }
.${state.id}-fo > div > strong { font-weight: 500; }
.${state.id}-g > g { cursor: pointer; }
`;
    return styleText;
  }

  function updateStyle() {
    svg.attr('class', addClass(svg.attr('class'), state.id));
    styleNode.text(getStyleContent());
  }

  function handleZoom() {
    const {
      transform
    } = d3.event;
    g.attr('transform', transform);
  }

  function initializeData(node) {
    let i = 0;
    let c = 0;
    const {
      colorDepth
    } = options;
    const container = document.createElement('div');
    const containerClass = `${state.id}-container`;
    container.className = addClass(container.className, `${state.id}-fo`, containerClass);
    const style = document.createElement('style');
    style.textContent = `
${getStyleContent()}
.${containerClass} {
  position: absolute;
  width: 0;
  height: 0;
  top: -100px;
  left: -100px;
  overflow: hidden;
  font: ${options.nodeFont};
}
.${containerClass} > div {
  display: inline-block;
}
`;
    document.body.append(style, container);
    walkTree(node, (item, next, parent) => {
      var _parent$p;

      i += 1;
      options.color(`${c}`); // preload colors

      const el = document.createElement('div');
      el.innerHTML = item.v;
      container.append(el);
      item.p = {
        // unique ID
        i,
        // color key
        c,
        el,
        ...item.p,
        // TODO keep keys for unchanged objects
        // unique key, should be based on content
        k: `${(parent == null ? void 0 : (_parent$p = parent.p) == null ? void 0 : _parent$p.i) || ''}.${i}:${item.v}`
      };
      next();
      if (!colorDepth || item.d === colorDepth) c += 1;
    });
    if (options.processHtml) options.processHtml(arrayFrom(container.childNodes));
    walkTree(node, (item, next) => {
      const rect = item.p.el.getBoundingClientRect();
      item.p.s = [Math.ceil(rect.width), Math.max(Math.ceil(rect.height), options.nodeMinHeight)];
      next();
    });
    container.remove();
    style.remove();
  }

  function setOptions(opts) {
    Object.assign(options, opts);
  }

  function setData(data, opts) {
    initializeData(data);
    state.data = data;
    if (opts) setOptions(opts);
    renderData(data);
  }

  function transition(sel) {
    if (options.duration) {
      return sel.transition().duration(options.duration);
    }

    return sel;
  }

  function fit() {
    const {
      width: offsetWidth,
      height: offsetHeight
    } = svgNode.getBoundingClientRect();
    const {
      minX,
      maxX,
      minY,
      maxY
    } = state;
    const naturalWidth = maxY - minY;
    const naturalHeight = maxX - minX;
    const scale = Math.min(offsetWidth / naturalWidth * options.fitRatio, offsetHeight / naturalHeight * options.fitRatio, 2);
    const initialZoom = d3.zoomIdentity.translate((offsetWidth - naturalWidth * scale) / 2 - minY * scale, (offsetHeight - naturalHeight * scale) / 2 - minX * scale).scale(scale);
    transition(svg).call(zoom.transform, initialZoom);
  }

  function handleClick(d) {
    var _data$p;

    const {
      data
    } = d;
    data.p = { ...data.p,
      f: !((_data$p = data.p) == null ? void 0 : _data$p.f)
    };
    renderData(d.data);
  }

  function renderData(originData) {
    var _origin$data$x, _origin$data$y;

    if (!state.data) return;
    const {
      spacingHorizontal
    } = options;
    const layout = d3Flextree.flextree().children(d => {
      var _d$p;

      return !((_d$p = d.p) == null ? void 0 : _d$p.f) && d.c;
    }).nodeSize(d => {
      const [width, height] = d.data.p.s;
      return [height, width + (width ? options.paddingX * 2 : 0) + spacingHorizontal];
    }).spacing((a, b) => {
      return a.parent === b.parent ? options.spacingVertical : options.spacingVertical * 2;
    });
    const tree = layout.hierarchy(state.data);
    layout(tree);
    adjustSpacing(tree, spacingHorizontal);
    const descendants = tree.descendants().reverse();
    const links = tree.links();
    const linkShape = d3.linkHorizontal();
    const minX = d3.min(descendants, d => d.x - d.xSize / 2);
    const maxX = d3.max(descendants, d => d.x + d.xSize / 2);
    const minY = d3.min(descendants, d => d.y);
    const maxY = d3.max(descendants, d => d.y + d.ySizeInner);
    state.minX = minX;
    state.maxX = maxX;
    state.minY = minY;
    state.maxY = maxY;
    if (options.autoFit) fit();
    const origin = originData ? descendants.find(item => item.data === originData) : tree;
    const x0 = (_origin$data$x = origin.data.x0) != null ? _origin$data$x : origin.x;
    const y0 = (_origin$data$y = origin.data.y0) != null ? _origin$data$y : origin.y; // Update the nodes

    const node = g.selectAll(childSelector('g')).data(descendants, d => d.data.p.k);
    const nodeEnter = node.enter().append('g').attr('transform', d => `translate(${y0 + origin.ySizeInner - d.ySizeInner},${x0 + origin.xSize / 2 - d.xSize})`).on('click', handleClick);
    const nodeExit = transition(node.exit());
    nodeExit.select('rect').attr('width', 0).attr('x', d => d.ySizeInner);
    nodeExit.select('foreignObject').style('opacity', 0);
    nodeExit.attr('transform', d => `translate(${origin.y + origin.ySizeInner - d.ySizeInner},${origin.x + origin.xSize / 2 - d.xSize})`).remove();
    const nodeMerge = node.merge(nodeEnter);
    transition(nodeMerge).attr('transform', d => `translate(${d.y},${d.x - d.xSize / 2})`);
    const rect = nodeMerge.selectAll(childSelector('rect')).data(d => [d], d => d.data.p.k).join(enter => {
      return enter.append('rect').attr('x', d => d.ySizeInner).attr('y', d => d.xSize - linkWidth(d) / 2).attr('width', 0).attr('height', linkWidth);
    }, update => update, exit => exit.remove());
    transition(rect).attr('x', -1).attr('width', d => d.ySizeInner + 2).attr('fill', d => options.color(d.data.p.c));
    const circle = nodeMerge.selectAll(childSelector('circle')).data(d => d.data.c ? [d] : [], d => d.data.p.k).join(enter => {
      return enter.append('circle').attr('stroke-width', '1.5').attr('cx', d => d.ySizeInner).attr('cy', d => d.xSize).attr('r', 0);
    }, update => update, exit => exit.remove());
    transition(circle).attr('r', 6).attr('stroke', d => options.color(d.data.p.c)).attr('fill', d => {
      var _d$data$p;

      return ((_d$data$p = d.data.p) == null ? void 0 : _d$data$p.f) ? options.color(d.data.p.c) : '#fff';
    });
    const foreignObject = nodeMerge.selectAll(childSelector('foreignObject')).data(d => [d], d => d.data.p.k).join(enter => {
      const fo = enter.append('foreignObject').attr('class', `${state.id}-fo`).attr('x', options.paddingX).attr('y', 0).style('opacity', 0).attr('height', d => d.xSize);
      fo.append('xhtml:div').select(function (d) {
        const node = d.data.p.el.cloneNode(true);
        this.replaceWith(node);
        return node;
      }).attr('xmlns', 'http://www.w3.org/1999/xhtml');
      return fo;
    }, update => update, exit => exit.remove()).attr('width', d => Math.max(0, d.ySizeInner - options.paddingX * 2));
    transition(foreignObject).style('opacity', 1); // Update the links

    const path = g.selectAll(childSelector('path')).data(links, d => d.target.data.p.k).join(enter => {
      const source = [y0 + origin.ySizeInner, x0 + origin.xSize / 2];
      return enter.insert('path', 'g').attr('d', linkShape({
        source,
        target: source
      }));
    }, update => update, exit => {
      const source = [origin.y + origin.ySizeInner, origin.x + origin.xSize / 2];
      return transition(exit).attr('d', linkShape({
        source,
        target: source
      })).remove();
    });
    transition(path).attr('stroke', d => options.color(d.target.data.p.c)).attr('stroke-width', d => linkWidth(d.target)).attr('d', d => {
      const source = [d.source.y + d.source.ySizeInner, d.source.x + d.source.xSize / 2];
      const target = [d.target.y, d.target.x + d.target.xSize / 2];
      return linkShape({
        source,
        target
      });
    });
    descendants.forEach(d => {
      d.data.x0 = d.x;
      d.data.y0 = d.y;
    });
  }
}

exports.markmap = markmap;
